﻿namespace TestProject;

/// <summary>
/// 프로그램
/// </summary>
class Program
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private
    ////////////////////////////////////////////////////////////////////// Event

    #region 클라이언트 메시지 도착시 처리하기 - client_MessageArrived(sender, message)

    /// <summary>
    /// 클라이언트 메시지 도착시 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="message">이벤트 인자</param>
    private static void client_MessageArrived(object sender, StreamingMessage message)
    {
        if(message.Data is StreamingDataToken token)
        {
            Console.Write(token.Message.Content);
        }
    }

    #endregion

    ////////////////////////////////////////////////////////////////////// Function

    #region 스트리밍 메시지 리스트 요청하기 (비동기) - RequestStreamingMessageListAsync(client)

    /// <summary>
    /// 스트리밍 메시지 리스트 요청하기 (비동기)
    /// </summary>
    /// <param name="client">API 클라이언트</param>
    /// <returns>태스크</returns>
    private static async Task RequestStreamingMessageListAsync(APIClient client)
    {
        client.MessageArrived += client_MessageArrived;

        List<Message> messageList =
        [
            new Message { Role = "system", Content = "당신은 서울 관광 전문 가이드 입니다."     },
            new Message { Role = "user"  , Content = "서울 북촌에 있는 맛집을 3곳 추천해주세요."}
        ];

        RequestMessage requestMessage = new RequestMessage();

        requestMessage.MessageList       = messageList;
        requestMessage.TopP              = 0.8d;
        requestMessage.TopK              = 0;
        requestMessage.MaximumTokenCount = 256;
        requestMessage.Temperature       = 0.5d;
        requestMessage.RepeatPenalty     = 5.0d;
        requestMessage.StopBefore        = new List<string>();
        requestMessage.IncludeAIFilters  = true;
        requestMessage.Seed              = 0;

        Console.WriteLine(messageList[1].Content);
        Console.WriteLine();

        List<StreamingMessage> streamingMessageList = await client.RequestStreamingMessageListAsync(requestMessage);
    }

    #endregion
    #region 스트리밍 메시지 비동기 열거 가능형 요청하기 (비동기) - RequestStreamingMessageAsyncEnumerableAsync(client)

    /// <summary>
    /// 스트리밍 메시지 비동기 열거 가능형 요청하기 (비동기)
    /// </summary>
    /// <param name="client">API 클라이언트</param>
    /// <returns>태스크</returns>
    private static async Task RequestStreamingMessageAsyncEnumerableAsync(APIClient client)
    {
        List<Message> messageList =
        [
            new Message { Role = "system", Content = "당신은 서울 관광 전문 가이드 입니다."     },
            new Message { Role = "user"  , Content = "서울 북촌에 있는 맛집을 3곳 추천해주세요."}
        ];

        RequestMessage requestMessage = new RequestMessage();

        requestMessage.MessageList       = messageList;
        requestMessage.TopP              = 0.8d;
        requestMessage.TopK              = 0;
        requestMessage.MaximumTokenCount = 256;
        requestMessage.Temperature       = 0.5d;
        requestMessage.RepeatPenalty     = 5.0d;
        requestMessage.StopBefore        = new List<string>();
        requestMessage.IncludeAIFilters  = true;
        requestMessage.Seed              = 0;

        Console.WriteLine(messageList[1].Content);
        Console.WriteLine();

        await foreach(StreamingMessage message in client.RequestStreamingMessageAsyncEnumerableAsync(requestMessage))
        {
            if(message.Data is StreamingDataToken token)
            {
                Console.Write(token.Message.Content);
            }
        }
    }

    #endregion
    #region 프로그램 시작하기 - Main()

    /// <summary>
    /// 프로그램 시작하기
    /// </summary>
    static async Task Main()
    {
        string clovaStudioAPIKey = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"; // X-NCP-CLOVASTUDIO-API-KEY
        string apiGatewayAPIKey  = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";                         // X-NCP-APIGW-API-KEY

        using APIClient client = new(clovaStudioAPIKey, apiGatewayAPIKey);

      //await RequestStreamingMessageListAsync(client);
        await RequestStreamingMessageAsyncEnumerableAsync(client);
    }

    #endregion
}